#/*!
# \file GeneR_glob.R
# \brief Base functions to access to GeneR buffers */
# 
# @History: <A. L.> 27 10 2005 
# Last modified: Time-stamp: <2006-02-10 17:40:32 lucas>
#
# ----------------------------------------------------



## --------------------
## @Function: abi.sys.free (seqno = 0)
## @+	     free.seq (seqno = 0)
## @Type: C
## @Purpose: internally free a buffer
## @+        returns seqno
## --------------------

freeSeq <- function(seqno = 0)
{
    .C("freeSeq", as.integer(seqno),PACKAGE="GeneR")[[1]]
}


## --------------------
## @Function: freeAllSeq ()
## @+	     freeAllSeq 
## @Type: C
## @Purpose: internally free all buffers
## @+        always returns 0
## --------------------

freeAllSeq  <- function()
{
   n <- nSeq()
     for(i in 0:(n-1))
        freeSeq(seqno=i)
   1  
}

## ---------------------------------------------------- 
## @Function: sizeSeq (seqno)
## @+	     sizeSeq			
## @Purpose:  retourne la longueur de la sequence contenue
## @+         dans le buffer seqno
## @Note:     < LC > add 15 Feb 00				
## ---------------------------------------------------- 

sizeSeq   <- function (seqno = 0)
{
	.C("size_buf",as.integer(seqno),as.integer(0),
           PACKAGE="GeneR") [[1]] -1
}
## --------------------
## @Function: .seqSize ()
## @+	     
## @Type: C
## @Purpose: returns sizes of internal buffers
## --------------------

.seqSize  <- function()
{
    n <- nSeq()
    seq <- comp <- rep(0,n)
    for(i in 0:(n-1))
      {
        seq[i+1] <- sizeSeq(seqno=i)
        comp[i+1] <- .C("size_buf",
                      as.integer(i),
                      as.integer(1),
                      PACKAGE = "GeneR")[[1]] - 1
      }
    
    list(sequences=seq,complementaires=comp)
    
}



## --------------------
## @Function: .seqalloc (size, seqno = 0, fill = 0)
## @+	     seqalloc (size, seqno = 0, fill = 0)
## @Type: C
## @Purpose: internally allocate a buffer
## @+        returns size dont forget to alloc last char \0
## --------------------

.seqalloc <- function(size, seqno = 0, fill = 1, comp=0)
{
    .C("myGeneR_alloc", as.integer(size),
    		    as.integer(seqno),
		    as.integer(fill),
       as.integer(comp),
       as.integer(-1),
       PACKAGE="GeneR")[[5]]
}


## --------------------
## @Function: .nseq()
## @+	     
## @Type: C
## @Purpose: returns number of internal buffers (or change this number)
## --------------------

nSeq  <- function(maxBuffers=NULL)
{
  if(is.null(maxBuffers))
    return(.C("sys_maxbuffer", as.integer(0),PACKAGE="GeneR"
       )[[1]])
  else
  .C("init_GeneR",as.integer(maxBuffers),PACKAGE="GeneR")    
}




