# ----------------------------------------------------
# @File:      ludglobal.R
# @Synopsis:  definit des variables globales pour R 
#
# @History:    8 Feb 00 created (Happy birthday !!!)
#
# ----------------------------------------------------


# --------------------
# @Function: abi.sys.transf.limites (limites,seqno)
# @+	     transf.limites
# @Type: R
# @Purpose:  Vrifie la compatibilit entre les vecteurs de positions d'origine
# @+         et les vecteurs de position de fin.
# @Note:    < LC > created  05 sep 00
# --------------------

.transf.limites <- function(seqno, limites)
{
  err <- 0
  
  ##print(list(liste = limites))
	
  if(sizeSeq(seqno) == 0)
    {
      warning("No sequences in buffer seqno")
      err <- -1
    }
		
  if ((is.list(limites)==FALSE) && (length(limites)!=2))
    {
      warning("error in the transformation of the limits")
      err <- -1
    }
		
  from <- as.integer(limites[[1]])
  to <- as.integer(limites[[2]])
  
##  to[to == 0] <- sizeSeq(seqno) -1
##  from[from == 0] <- sizeSeq(seqno) -1
##  from[from == 1] <- getBegSeq(seqno)
##  to[to == 1] <- getBegSeq(seqno)
  
  if(length(from) != length(to))
    {
      warning("from and to must be of equal size")
      err <- -1
    } 
	
 ## from  <- adrInput(adr = from, seqno = seqno)
 ## to <- adrInput(adr = to, seqno = seqno)
		
  if ((getStrand() == 1) )
    {
      ## now this is included in C...
      ##if (revComp(seqno) == -1)
      ##  return (-1)
      adr <- from
      from <- AtoT(to, seqno =seqno )
      to <- AtoT(adr, seqno =seqno)
    }
  else
    {
      from <- AtoT(from, seqno =seqno )
      to <- AtoT(to, seqno =seqno)
    }

  
  if(! any(is.na(from)) && ! any(is.na(to)))
    {
      if (all(from<=to)==FALSE)
        {
          warning("error: from must be <= to")
          err <- -1
        }
      
      if(  (all(from != -1)==FALSE) || (all(to!=-1)==FALSE)  )
        {
          warning("error with from and to")
          err <- -1
        }
    }
  else
    {
      warning("NA in from or To")
      err <- -1
    }
  lst <- list (err,from, to)
  
  lst
         
}


# --------------------
# @Function: lud.glo.setstrand (str = 0)
# @Type: C
# @Purpose:  affecte Strand 
# @+         variable globale declarant sur quel brin on travaille
# @+         0 : brin sens
# @+         1 : brin complementaire
# @+         renvoie NULL si erreur
# --------------------

setStrand <- function(str = 0)
{
   if((str == 0 ) || (str == 1))
   return(.C("setStrand",as.integer(str),PACKAGE="GeneR") [[1]])
  else
    {
      warning("Strand must be 0 or 1")
      return(NULL)
    }
 }

# --------------------
# @Function: lud.glo.isstrandsens (): now getStrand
# @Type: C
# @Purpose:  lit Strand et retourne 
# @+                                0  si brin sens
# @+                                1  si brin complementaire
# @+                                2  si brin aleatoire
# @+                                -1 si non defini
# --------------------

getStrand   <- function()
{
	.C("getStrand",as.integer(-1),PACKAGE="GeneR") [[1]]
}

# --------------------
# @Function: lud.glo.wbegseq (pos = 1, seqno=0)
# @Type: C
# @Purpose:  affecte BegSeq
# @+         variable globale contenant la position du debut de la sequence
# --------------------

setBegSeq  <- function(pos = 1, seqno = 0)
{
  res <- getParam(seqno)
  if(is.null(res))
    return(NULL)
  
  .C("setParam",
     seqno=as.integer(seqno),
     as.integer(pos),
     end=res[3],
     err=as.integer(-1),
     PACKAGE="GeneR")[[4]]
}

# --------------------
# @Function: lud.glo.rbegseq (seqno = 0)
# @Type: C
# @Purpose:  retourne la valeur de BegSeq
# --------------------

getBegSeq <- function(seqno = 0)
{
  res <- getParam(seqno)
  if(is.null(res))
    return(NULL)
  res[1]
}


# --------------------
# @Function: lud.glo.rseqsize (seqno = 0)
# @Type: C
# @Purpose:  initialiase la valeur "masterSize"
# --------------------

setSeqSize <- function(pos = sizeSeq,seqno = 0)
{
  res <- getParam(seqno)
  if(is.null(res))
    return(NULL)
  
  .C("setParam",
     seqno=as.integer(seqno),
     begin=res[1],
     as.integer(pos),
     err=as.integer(-1),
     PACKAGE="GeneR")[[4]]
  
}

# --------------------
# @Function: lud.glo.rseqsize (seqno = 0)
# @Type: C
# @Purpose:  retourne la valeur de SeqSize
# --------------------

getSeqSize <- function(seqno = 0)
{
  res <- getParam(seqno)
  if(is.null(res))
    return(NULL)
  res[3]
}


getEndSeq <- function(seqno=0)
  {
    getBegSeq(seqno) + sizeSeq(seqno) - 1
  }

# --------------------
# @Function: lud.glo.waccn (name, seqno = 0)
# @Type: C
# @Purpose:  affecte AccN
# @+         variable globale contenant le numero d'accession de la sequence
# --------------------

setAccn <- function(name, seqno = 0)
{
  lst <- .C("setAccno",as.integer(seqno),
            name=as.character(name),err=as.integer(0),
            PACKAGE="GeneR")
	
  if(!lst$err)
    {
      warning("error")
      return(NULL)
    }
  lst$name
}

# --------------------
# @Function: lud.glo.raccn (seqno = 0)
# @Type: C
# @Purpose:  retourne AccN
# --------------------

getAccn <- function(seqno = 0)
{
  ## Name of sequences are stored with maximum: 65 chars.
  name <- .Call("alloc_char",as.integer(65),as.integer(1),PACKAGE="GeneR")

  lst <- .C("getAccno",as.integer(seqno),
            name=name,err=as.integer(0),
            PACKAGE="GeneR")
	
  if(!lst$err)
    {
      warning("error")
      return(NULL)
    }
  lst$name
}

# --------------------
# @Function: lud.glo.wparam (strand = 0,mode = 0,beg = 1,end,size,accn, seqno  =0)
# @Type: C
# @Purpose:  affecte ces 3 parametres
# @+         
# @+         retourne -1 si end < beg
# @+                  ou si size < end
#                       0 errreur, 1 OK.
# --------------------

setParam <- function(from = 1,sizeMaster=seqSize(), seqno=  0)
{
  size <- sizeSeq()
  to <- from + size - 1
	if(((to < from) && (to != 0)) || (size < to) || (size < from))
		return (-1)
  .C("setParam",
     seqno=as.integer(seqno),
     begin=as.integer(from),
     sizeMaster=as.integer(sizeMaster),
     err=as.integer(-1),
     PACKAGE="GeneR")[[4]]
}

# --------------------
# @Function: lud.glo.rparam ()
# @Type: C
# @Purpose:  retourne les 3 parametres
# @+         les renvoie sous forme de vecteur
# --------------------

getParam <-  function(seqno=0)
{
  res<- .C("getParam",
           seqno=as.integer(seqno),
           begin=as.integer(-1),
           size=as.integer(-1),
           Mastsize=as.integer(-1),
           err=as.integer(-1),          
           PACKAGE="GeneR")
  if(!res$err)
    return(NULL)
  else
    c(res$begin,res$size,res$Mastsize)
}

# --------------------
# @Function: lud.glo.AtoR (adrabs,seqno= 0)	
# @Purpose:  transforme les adresses absolues contenues
# @+         dans le vecteur adrabs en
# @+         adresses relatives 
# --------------------

RtoA <- AtoR <- function (x,seqno=0)
{
  limites <- getParam(seqno)
  ## 0 and 1 are special...
  x[x==0] <- limites[1]+limites[2]-1
  x[x==1] <- limites[1]
  x[x < limites[1]]  <- NA
  x[x > (limites[1]+limites[2]-1)  ]  <- NA
  if(!getStrand())
    return(x)

  limites[3] - x +1

}


# ---------------------------------------------------- 
# @Function: lud.glo.AtoT (adrabs,seqno=0)		 	
# @Purpose:  transforme les adresses absolues en	
# @+         adresses veritables			 
# ---------------------------------------------------- 

AtoT <- function (x,seqno=0)
{
  limites <- getParam(seqno)
  endseq <- limites[1] + limites[2] - 1
  ## 0 and 1 are special...
  x[x==0] <- endseq
  x[x==1] <- limites[1]
  x[x < limites[1]]  <- NA
  x[x > endseq ]  <- NA
  if(!getStrand())
    return(x- limites[1] +1)
  else
    return(endseq - x + 1)

}

# ---------------------------------------------------- 
# @Function: lud.glo.TtoA (adrver, seqno=0)		 	
# @Purpose:  transforme les adresses veritables en	
# @+         adresses absolues 			
# ---------------------------------------------------- 

TtoA <- function (x,seqno=0)
{
  limites <- getParam(seqno)
  x[x < 0]  <- NA
  x[x> limites[2] ] <- NA
  x[x==0] <- limites[2]
  if(!getStrand())
    return(x + limites[1] -1)
  else
    return(limites[1] + limites[2] - x)      
  
  
}

# ---------------------------------------------------- 
# @Function: lud.glo.RtoT (adrrel,seqno=0)		 	
# @Purpose:  transforme les adresses relatives en	
# @+         adresses veritables 			
# ---------------------------------------------------- 

RtoT <- function (x,seqno=0)
{
  limites <- getParam(seqno)
  ## 0 and 1 are special...
  x[x==0] <- limites[1]+limites[2]-1
  x[x==1] <- limites[1]
  x[x < limites[1]]  <- NA
  x[x > (limites[1]+limites[2]-1)  ]  <- NA
  if(!getStrand())
    return(x- limites[1] +1)
  else
    return(x- limites[3] + limites[1] + limites[2] - 1 )
  
}

# ---------------------------------------------------- 
# @Function: lud.glo.TtoR (adrver, seqno=0)		 	
# @Purpose:  transforme les adresses veritables en	
# @+         adresses relatives 			
# ---------------------------------------------------- 

TtoR <- function (x, seqno=0)
{
  limites <- getParam(seqno)
  x[x < 0]  <- NA
  x[x> limites[2] ] <- NA
  x[x==0] <- limites[2]
  if(!getStrand())
      return(x + limites[1] -1)
  else
      return(x + limites[3] - limites[1] - limites[2] + 1)      

}

 


