bankDensityProfile <- function (file, seqno=0, fun=seqSkew, fileout= NULL, nbin, sizeBin,allSeq=FALSE,fromEnd=FALSE, name = "all",threshold=0,strand=getStrand(),accu=FALSE,case="all")

#----------------------------------------------------------------------------------
# bankDensityProfile
#
# Description :
#
# computes mean, standard deviation and number of accumulations of bias in sub-segments for a set of segments in a bank
#
# Usage :
#
# bankDensityProfile (ori, from, to, seqno, fun="seqSkew", fileout= NULL, nbinL, nbinR, sizebin)
#
# Arguments :
#
# ori       integer, the position of the origin
# from      integer, position of the begin of segment
# to        integer, position of the end of segment
# seqno     integer, sequence number
# fun       function to be used
# fileout
# nbinL     number of sub-segments to create before the origin
# nbinR     number of sub-segments to create after the origin
# sizebin   integer, size of the sub-segments
#
# Value :
#
# data frame
# [1]  begin of sub-segments
# [2]  mean (matrix) of bias in sub-segments
# [3]  standard deviation (matrix) of bias in sub-segments
# [4]  number of accumulations (matrix) of bias in sub-segments
#
# Call :
#
# accuDensity
#
# author :
#
# Emna
# 07 Oct 2005
#----------------------------------------------------------------------------------------

  
  {

    ## Accumulation can be take into account if function is a subfunction...
    ## else <<- operator will store data in global environement
    prev.accudensity <- function (event, before=1,after=1,size=1,fun=fun,threshold=0,case="all",fileout)
      {
        
      
        A <- .accuDensity  (event=event, before=before,after=after,size=size,fun=fun,threshold=threshold,case=case,seqno=0,Xacc)
      
        
        ## tmp: when binPos are possible AND when A was computed (because of the N)
        ##     tmp<- ( is.na(binPos[,1]) + is.na(A[,1])) == 0
        if(!is.null(A))
           tmp<- ! is.na(A[,1])     
        else
          return (1)

        if(sum(tmp) == 0)
          return (1)

        ## If file: append to file but just first column
        if(!is.null(fileout))
           write.table(t(A[,1]),file=fileout,col.names=FALSE,sep='\t',row.names=FALSE,append=TRUE)

           


          
        if(is.null(Xacc))
          {
            if(!is.null(A))
              {
                Xacc<<-A
                Xacc[!tmp,] <<- 0
                X2acc<<-A^2
                X2acc[!tmp,] <<- 0
                Nacc<<- !is.na(A[,1]) + 0
              }
          }
        else
          {
            Xacc[tmp,] <<-Xacc[tmp,]+A[tmp,]    
            X2acc[tmp,] <<- X2acc[tmp,]+(A[tmp,]^2)          
            Nacc[tmp] <<- Nacc[tmp]+1
          }

        return (1)      
      }

    ## Begining of function bankDensityProfile
    
        
    y <- rep (0, nbin)
    Xacc <- NULL
    X2acc <- Xacc
    Nacc <- Xacc
    
    if(name[1]=="all")
      {
        makeIndex(file,type="F")
        ids <- as.character(read.table(paste(file,".ix",sep=""))[,1])
      }
    else
      ids <- name

    if(length(strand)== 1)
      strand <- rep(strand,length(ids))

    
    for (i in 1:length(ids))
     {
       readFasta(file,name=ids[i])
       if(!fromEnd) ## i.e from begining
         {
           nbinL <- 0
           nbinR <- nbin
           if(allSeq)
             to = sizeSeq()
           else
             to = sizeSeq()/2
           event <- c(0,1,to,strand[i])
         }
       else
         {
           nbinL <- nbin
           nbinR <- 0
           if(allSeq)
             from = 1
           else
             from = sizeSeq()/2
           event <- c(sizeSeq(),from,sizeSeq(),strand[i])

         }
       prev.accudensity (event, before=nbinL,after=nbinR,size=sizeBin,fun=fun,case=case,fileout=fileout)

     }

    
    
    if(accu)
      return(list(sum=Xacc,sumsquare=X2acc,count=Nacc))

    moy <- Xacc/Nacc
    
    sigma <- sqrt ((X2acc-((Xacc^2)/Nacc))/Nacc)/sqrt((Nacc-1)*(Nacc >0) )
          
    begin <- seq (-nbinL*sizeBin, nbinR*sizeBin-1 , by=sizeBin)

    res <- list()
  
    for(i in 1:ncol(Xacc))
      {
        res[[i]] <- cbind(x=begin,y=moy[,i],dy=sigma[,i],count=Nacc)
        names(res)[i]<- dimnames(Xacc)[[2]][i]
        class(res[[i]])<- "profile"
      }
    
    return (res)
    
  }



