


# --------------------
# @Function: abi.str.composeq (s,wsize=1)
# @Type: C
# @Purpose: donne la composition en mono, di ou
# @+        tri nuclotides de s
# @Note:    < LC > add 15 Mar 00
# @Note:    < AL > Modified Feb 2005
# --------------------

compoSeq <- function(seqno=0,from=1,to=0,strand=getStrand(),
                     wsize=1,p=TRUE,case="all",step=wsize)
{
  if(length(strand)> 1)
    {
      warning("Only first element of strand will be used")
      strand <- strand[1]
    }
  lestrand <- getStrand()

  setStrand(strand)
  l <- 5^wsize
  if(wsize>15)
    {
      warning("wsize >15, to large !")
      return(NULL)
    }
  CASE <- c("all","upper","lower")
  case <- pmatch(case, CASE)

  if(length(from) != length(to))
    {
      warning("from and to must be of same length")
      return(NULL)
    }
  ## two verifs...
  if(sizeSeq(seqno) == 0 )
    return(NULL)
  if(sum(strand>0))
    revComp()

  naindices  <- (is.na(from) + is.na(to)>0)
  from <- from[!naindices]
  to <- to[!naindices]
  
  longeurtableau <- as.integer(length(from))

  ##addrs <- apply(cbind(from,to,strand),1,function(u){setStrand(u[3]);tmp <- AtoT(seqno=seqno,c(u[1],u[2]));tmp})
  ##from <- addrs[1,]
  ##to <- addrs[2,]
  limites <- list(from,to)

  liste <- .transf.limites(seqno, limites)
	
  if(liste[[1]] == -1)
    {
      warning("error in the transformation of the limits")
      return (NULL)
    }
  
  from <- liste[[2]]
  to <- liste[[3]]

  
  lst <- .C("composeq",
            as.integer(seqno),
            as.integer(from),
            as.integer(to),
            as.integer(strand),
            longeurtableau,
            as.integer(length(strand)),
            as.integer(wsize),
            lab=rep("               ",l),
            res=integer(l*longeurtableau),
            as.integer(step),
            case,
            err=as.integer(0),
            PACKAGE="GeneR")
  
  if (lst$err<0)
    return(lst$err)
  
  dat <- matrix(lst$res,nrow=longeurtableau,byrow=TRUE)
  colnames(dat) <- lst$lab

  ## Pourcentage
  if(p==TRUE)
    if(sum(dat)>0)
      dat <- t(apply(dat,1,function(u){if(sum(u)>0){ u/sum(u)}}))

  setStrand(lestrand)

  if(sum(naindices)>0)
    {
      ## We put NA lines, where the from and to had NA.
      newmat <- rbind(dat,matrix(rep(0,ncol(dat)*sum(naindices)),ncol=ncol(dat)))
      newmat[!naindices,]<- dat
      newmat[naindices,]<- NA
      return(newmat)
    }
  else
    return(dat)

  
}


# --------------------
# @Function: abi.str.composeq (s,wsize=1)
# @Type: C
# @Purpose: donne la composition en mono, di ou
# @+        tri nuclotides de s
# @Note:    < LC > add 15 Mar 00
# @Note:    < AL > Modified Feb 2005
# --------------------

strCompoSeq <- function(s,wsize=1,p=TRUE,case="all",step=wsize)
{
  l <- 5^wsize
  if(wsize>15)
    {
      print("wsize >15, to large !")
      return(-1)
    }

  ## case = 1 (all),2 (upper) ,3 (lower)
  CASE <- c("all","upper","lower")
  case <- pmatch(case, CASE)

  lst <- .C("strcomposeq",
            as.character(s),
            res=integer(l),
            as.integer(wsize),
            lab=rep("               ",l),
            as.integer(1),
            as.integer(step),
            case,
            err=as.integer(0),
            PACKAGE="GeneR")
  
  if (lst$err<0)
    return(lst$err)
  
  dat <- t(lst$res)
  colnames(dat) <- lst$lab
  
  if(p==TRUE)
    if(sum(dat)>0)
      dat <- dat /sum(dat)
  
  dat
	
}




seqSkew <- function(seqno=0, from=1,to=0,strand=getStrand(),case="all")
{
# fonction modifie par emna le 29sept05 : remplacement de data.frame
# par segSet 
#-----------------------------------------------------------------
# seqSkew
#
# Description
#
# compute the strand assymetries from one sequence fragment
#
# Usage
#
# seqSkew <- function(seqno=0, segment)
#
# Arguments:
#
# seqno         integer, sequence number
# segment	segSet, [,1] begin [,2] end of segment
# 
# Value:
#
# data frame
# [1] TA skew : (T-A)/(T+A)
# [2] GC skew : (G-C)/(G+C)
# [3] total skew : TA+GC skews
# [4] number of ambigous nucleotides
#
# Note
#:
# N's are ignored
#

# author :
#
# Yves
# 18 Jan 2005
#------------------------------------------------------------------


  compos <- compoSeq(seqno=seqno, from=from, to=to, strand=strand, p=0,case=case) 


  if (!is.na(compos[1]))
    {
      if (compos[1] == -1)
        { 
          print ("empty seq buffer")
          return (NULL)
        }
    }
  ## print (compos)

  W <- compos[,3]+compos[,1]

  W[W==0] <- 1
                                          

  S <- compos[,4]+compos[,2]

  S[S==0] <- 1 
  

  skta <- (compos[,1]-compos[,3])/W
  skgc <- (compos[,4]-compos[,2])/S
  sktot <- skta+skgc


  skew <- cbind(skta, skgc, sktot,N=compos[,5])

  return(skew)
 
}



GCcontent <- function(seqno=0, from=1,to=0,case="all",...)
{
# fonction modifie par emna le 29sept05 : remplacement de data.frame
# par segSet 
#-----------------------------------------------------------------
# seqSkew
#
# Description
#
# compute the strand assymetries from one sequence fragment
#
# Usage
#
# seqSkew <- function(seqno=0, segment)
#
# Arguments:
#
# seqno         integer, sequence number
# segment	segSet, [,1] begin [,2] end of segment
# 
# Value:
#
# data frame
# [1] TA skew : (T-A)/(T+A)
# [2] GC skew : (G-C)/(G+C)
# [3] total skew : TA+GC skews
# [4] number of ambigous nucleotides
#
# Note
#:
# N's are ignored
#

# author :
#
# Yves
# 18 Jan 2005
#------------------------------------------------------------------


compos <- compoSeq(seqno=seqno, from=from, to=to, strand=0, p=1,case=case) 


if (!is.na(compos[1]))
  {
  if (compos[1] == -1)
   { 
     print ("empty seq buffer")
     return (NULL)
   }
  }
# print (compos)


to[to==0] <- sizeSeq(seqno)
pgc <- (compos[,4]+compos[,2])
N <- compos[,5] * (to-from+1)


return(cbind(pgc,N))
 
}




	


# ---------------------------------------------------- 
# @Function: abi.sys.exact.word(seqno,word)	
# @+	     exact.word
# @Purpose:  renvoie la position de word dans  	
# @+	      la sequence situe dans le buffer seqno	
# @+	     renvoie 0 si pas trouv
# ---------------------------------------------------- 

exactWord  <- function(word, seqno = 0, from = 1, to = 0, strand =
                 getStrand(), step = 1, overlap=TRUE,wNbOcc = -1, case.sensitive =
                 FALSE)

{
  lestrand <- getStrand()
  if(sizeSeq(seqno) <= 1)
    {
      warning("No sequences in buffer seqno")
      return(list(NULL))
    }

  if(length(strand) == 1)
    strand <- rep(strand,length(from))
    


  
##  print(list(from=from, to=to))  
  
  tab <- array(c(from,to,strand), dim=c(length(from),3))


  
  lst <- unlist(apply(tab,1,list),recursive = FALSE)


  
  lst <- lapply(lst, .exactmatch,seqno=seqno, word = word, wNbOcc=wNbOcc,
         step=step,overlap=overlap,case.sensible=case.sensitive) 


  setStrand(lestrand)

  lst
}

# -------------------------------------------------------------------
# exactmatch : fonction appele par exact.word
# -------------------------------------------------------------------

.exactmatch  <- function(seqno=0,word,limites,wNbOcc,step,overlap=TRUE,case.sensible=FALSE)
  {
	
    from <- limites[[1]]
    to <- limites[[2]]
    strand<- limites[[3]]
    setStrand(strand)
    
    limites <- list(from,to)
    liste <- .transf.limites(seqno, limites)
	
    if(liste[[1]] == -1)
      {
        warning("error in the transformation of the limits")
        return (NULL)
      }
  
    from <- liste[[2]]
    to <- liste[[3]]
    
    if (wNbOcc==-1)           # Nombre d'occurences desirees pas definie
      {
        if(to > from)
          wNbOcc <- to-from+1
        else
          wNbOcc <- from-to+1
      }

    
    lst<-.C("sys_exactmatch",as.integer(seqno),
            as.character(word),as.integer(from),
            as.integer(to), as.integer(strand),as.integer(wNbOcc),
            as.integer(step),
            integer(wNbOcc),as.integer(0),
            as.integer(case.sensible),
            as.integer(overlap),
            as.integer(0),PACKAGE="GeneR")



    
    if (lst[[10]]==-1)	
      {
        warning("error")
        return (-1);
      }
    
    res <- lst[[8]]

    
    nocc <- lst[[9]]    # nb occurences
    
    if(nocc == 0)
      return(-1)
    
    res <- res[1:nocc]
    

    res <- TtoA(x = res, seqno =  seqno)
    if(strand)
      res <- res - nchar(word) +1

    
    return(res)
    
  }  
