sliceSegment <-  function(from=1, to=0, ori=0, nbinL=0, nbinR=0, size=1)
   
# fonction modifie le 28sep05 par emna : suppression de seqno 
#-----------------------------------------------------------------
# sliceSegment
#
# Description:
#
# delineate sub-segments of equal size at both sides of an origin in a segment
#
# Usage:
#
# sliceSegment (from=1, to=1, origin = 1, before=0, after=0, size=1)
#
# Arguments:
# 
# from 		integer, position of the begin of segment
# to		integer, position of the end of segment
# size		integer, size of the sub-segments
# before        number of sub-segments to create before the origin
# after         number of sub-segments to create after the origin
# ori	integer, the position of the origin
# 
# Value :
#
#	data frame
# [1]  begin of sub-segments
# [2]  end of sub-segments
#
# Notes:
#
# In the "to" argument, 0 does not mean end of fragment.
# All sub-segments outside the segment are set to NA.
# Position of the origin is the end of the last "before" sub-segment.
# 
# author:
#
# Yves d'Aubenton
# 9 March 2005
#------------------------------------------------------------------
{
    a <- ori+1
    b <- nbinR*size + ori
    if (a <= b)
    {
        begafter <- seq(from=a, to=b, by=size)
        begafter[begafter > to-size+1] <- NA
    }
    else
    {
        begafter <- rep.int(NA, times=nbinR)
    }
    
    a <- ori-(nbinL*size)+1
    b <- ori
    if (a <= b)
    {
        begbefore <- seq(from=a, to=b, by=size)
        begbefore[begbefore < from] <- NA
    }
    else
    {
        begbefore <- rep.int(NA, times=nbinL)
    }        
    
    begin <- c(begbefore,begafter)
    end <- begin+size -1
    
    return(data.frame(begin,end))
}
   

## Plot a profile.
plot.profile <- function(x,ylim=NULL,...)
  {
    
    xx <- x
    x  <- xx[,1]
    y  <- xx[,2]
    dy <- xx[,3]
    ymax <- max(c(y+dy,-(y-dy),y,-y),na.rm=TRUE)
    delY <- c(-ymax,ymax)
    if(!is.null(ylim))
      delY <- ylim
    
    plot(x,y,type="l",ylim=delY,...)

    ## Error bars
    points(x,y+dy,pch='_',col='gray')
    points(x,y-dy,pch='_',col='gray') 

    i <- 1
    lapply(x,function(u){lines(c(u,u),c(y[i]-dy[i],y[i]+dy[i]),col='gray'); i <<- i+1})
    1

  }

.accuDensity <- function (event, before=1,after=1,size=1,fun=fun,threshold=0,case="all",seqno=0,Xacc)

#------------------------------------------------------------------------------------
# accuDensity
#
# Description :
#
# 
#
# Usage :
#
# accuDensity (event)
#
# Arguments :
#
# event  
# 
# Value :
#
# (-1)   error
# (0)    program completed
#
# Call :
#
# sliceSegment, seqSkew
#
# author :
#
# Emna
# 30 Sept 2005
#-------------------------------------------------------------------------------------
  {
     ori <- event[1]
     from <- event[2]
     to <- event[3]
     strand=event[4]

     binPos <- sliceSegment (ori=ori,from=from,to=to,nbinL=before,nbinR=after, size=size)

#      print (Xacc) 
#      print(binPos)

     if(sum(is.na(binPos[,1])) != nrow(binPos))
       A <- fun (seqno=seqno, from=binPos[,1],to=binPos[,2],strand=strand,case=case)
     else
       {
         if(is.null(Xacc))
           return(NULL)
         else           
           return(matrix(NA,nrow=nrow(binPos),ncol=ncol(Xacc)))
         
       }
         
     if (is.null(A))
        {
         warning ("Error with function fun.")
         return (NULL)
        }
     Ncount <- A[,ncol(A)]
     id <- dimnames(A)[[2]][-ncol(A)]
     A <- A[,-ncol(A)]
     if(is.null(dim(A)))
       A[Ncount>threshold] <- NA
     else
       A[Ncount>threshold,] <- NA
     
     ## Matrix of size 1: changed to vectors... so re-change to matrix
     A <- data.frame(matrix(A,nrow=nrow(binPos)))
     names(A) <- id
     A
   }

