## ----------------------------------------------------
## @File: makeindex.R
## @Synopsis: All functions related to index files
##            of Embl, Gbk & Fasta files
##
## @History: <Gloup> 10 Feb 2005 created by A. LUCAS
## Last modified: Time-stamp: <2006-03-09 12:04:28 lucas>
##
## ----------------------------------------------------

## ----------
## Fuction indexFasta
## make (even if it already exists) a new index for
## file
## ----------
indexFasta <- function(file)
  { 
    
    lines <- readLines(file ,n=1)
    if(substr(lines,1,1) != ">")
      {
        warning("Fasta file not recognised, must begin with '>'")
        return(-1)
      }
    res <- .C("ixfasta",
              as.character(file),
              res=as.integer(0),
              PACKAGE="GeneR"
              )$res
    if(res == -1)
      return(-1)
    if(res == -2)
      warning("Carriage return found; should use function deleteCR on your bank file")
     if(res == -3)
      warning("Some access number are too long")
     if(res == -4)
       {
         warning("Could not open file")
         return(-1)
       }
   lines <-readLines( paste(file,'.ix',sep='') )
    if(length(lines) == 0)
      return (-1)
    index <-read.table(paste(file,'.ix',sep='') )[,1]
    lines <- lines[order(.str2ascii(index))]
    write(lines,file=paste(file,'.ix',sep=''))
    1
  }

## ----------
## Fuction indexGbk
## make (even if it already exists) a new index for
## file
## ----------
indexGbk <- function(file)
  {
    fileix <- paste(file,'.ix',sep='')

    lines <- readLines(file ,n=1)
    if(substr(lines,12,12) != " ")
      {
        warning("Gbk file not recognized")
        return(-1)
      }

    res <- .C("ixgbk",
              as.character(file),
              res = as.integer(0),
              PACKAGE="GeneR"
              )$res
    if(res == -1)
      return(-1)
    if(res == -2)
      warning("Carriage return found; should use function deleteCR on your bank file")
     if(res == -3)
      warning("Some access number are too long")
     if(res == -4)
       {
         warning("Could not open file")
         return(-1)
       }
    lines <-readLines( fileix )
    if(length(lines) == 0)
      return (-1)
    index <-read.table(fileix)[,1]
    lines <- lines[order(.str2ascii(index))]
    write(lines,file=fileix)
    1
  }

## ----------
## Fuction indexEmbl
## make (even if it already exists) a new index for
## file
## ----------
indexEmbl <- function(file,index="ix")
  {


    lines <- readLines(file ,n=1)
    if(substr(lines,3,5) != "   ")
      {
        warning("Embl file not recognised, must begin with 2 chars and 3 spaces like 'ID   FOO'")
        return(-1)
      }
    res <- .C("ixembl",
              as.character(file),
              as.character(substr(index,2,2)),
              res=as.integer(0),
              PACKAGE="GeneR"
              )$res
    if(res == -1)
      return(-1)
    if(res == -2)
      warning("Carriage return found; should use function deleteCR on your bank file")
     if(res == -3)
      warning("Some access number are too long")
    if(res == -4)
      {
        warning("Could not open file")
        return(-1)
      }
    lines <-readLines( paste(file,'.',index,sep='') )
    if(length(lines) == 0)
      return (-1)
    liste <-read.table(paste(file,'.',index,sep='') )[,1]
    lines <- lines[order(as.character(liste))]
    write(lines,file=paste(file,'.',index,sep=''))
    1
  }


## ---------------------------
## @Function : makeIndex (bankpath, type="E")
## @Type : R
## @Purpose : Cree le fichier index correspondant au fichier bankname
## @+         do not make index if file already exists
## @Note : <LC> created 08-10-01. modified 10-Feb-05
## ---------------------------

makeIndex <- function(file, type="E", index="ix")
  {
    fileix <- paste(file,index,sep=".")
    if(file.exists(file) == FALSE)
      {
        warning(paste("File", file, "does not exists"))
        return(-1)
      }

    if(type != "E" && type != "F" && type != "G")
      {
        warning("Bank type not valid.\nE : Embl.\nF : Fasta.\nG : GenBank.\n")
        return (-1)
      }

    # Si l'index est deja fait, et qu'il est plus recent que le fichier c'est fini !!!
    if(file.exists(fileix) == TRUE)
      {
       if( file.info(fileix)$mtime >  file.info(file)$mtime)
        return(1)
      }

    if(type == "E")
      {
        if(index != "ix" && index != "id")
          {
            warning("Index extension not valid (ix or id)")
            return(-1)
          }
        return(indexEmbl(file,index))
      }
    
    if(type == "F")
      {
        if(index != "ix")
          {
            warning("Index extension not valid (ix)")
            return(-1)
          }
        return(indexFasta(file))
      }

    if(type == "G")
      {
        if(index != "ix")
          {
            warning("Index extension not valid (ix)")
            return(-1)
          }
        return(indexGbk(file))
      }

    return (0)
  }


# --------------------
# @Function: abi.sys.paramFromIndex (accn,indexpath) (former: xtoffsetseq) 
# @+	     paramFromIndex
# @Type: C
# @Purpose:  cherche dans le fichier indexpath		
# @+         les parametres suivants correspondant  l'accn donn :
# @+         offset features, offset sequence, la taille de l'entree 
# @Note:     < LC > add Feb 00
# --------------------

.paramFromIndex <- function(accn,indexpath)
{
  .C("XtLigIxC",
          accno=as.character(accn),
          indexpath=as.character(indexpath),
          beginFeatures=as.integer(0),
          beginSeq=as.integer(0),
          size=as.integer(0),
          PACKAGE="GeneR")
}



## delete \r in file
## return 1: ok; 0: proble while create file
deleteCR <- function (file)
  {
    file <- as.character(file)
    filetmp <- paste(file,floor(runif(1,1,50000)),".tmp",sep="")

    ## delete Carriage return in bank file
    err <- .C("delete_CR_infile",
              file,
              filetmp,
              err=as.integer(1),
              PACKAGE="GeneR")$err

    if(err == -1)
      {
        warning("could not access to file or create a temporary file\n")
        return(0)
      }
    
    file.rename(filetmp,file)
    
    return(1)

  }
