.allOrfs1phase <- function (phase=1,seqno=0,start="atg",stop=c("taa","tag","tga"),suborfs=TRUE)
{
  From <- NULL
  To <- NULL
  phase <- phase + getBegSeq(seqno) -1
  for(i in 1:length(start)){
    new<-exactWord(seqno=seqno, word=start[i], from=phase, step=3)[[1]]
    if(is.null(new)){
      return(NULL) }
    else From <- c(From,new)       
  }
  for(i in 1:length(stop)){
    new<-exactWord(seqno=seqno, word=stop[i], from=phase, step=3)[[1]]
    if(is.null(new)){
      return(NULL) }
    else To <- c(To,new)
  }

  start <- From
  stop <- To 

  start <- sort(start)
  while(start[1]== -1) {
    start <- start[-1]
    if (length(start)==0)
      return(NULL)
    }
  
  start <-start[start<max(stop)]
  if(length(start)>0)
    {
      stop <- sort(stop+2)
      stop <- sapply(start,function(u){min(stop[stop>u],na.rm=FALSE)})
      if(suborfs)
        return(cbind(start,stop))
      else
        return(unionSeg(cbind(start,stop)))
    }
  else
    NULL
}
#####################################################################################

.allOrfs1phaseComplete <- function (phase=1,seqno=0,start="atg",stop=c("taa","tag","tga"),suborfs=TRUE)
{

#------------------------------------------------------------------------------------
# .allOrfs1phaseComplete
#
# Description :
#
# Gets incomplete Orfs in a sequence for a given phase
#
# Usage :
#
# .allOrfs1phaseComplete (phase=1,seqno=0,start="atg",stop=c("taa","tag","tga"))
#
# Arguments :
#
# phase      integer/scalar, 1,2 or 3
# seqno      integer/scalar, Sequence number (buffer number)
# start      string/vector, start codons
# stop       string/vector, stop codons
#
# Value :
#
# .allOrfs1phaseComplete returns a table of positions
#
# Author
#
# Emna
# 24 Nov 2005
#-------------------------------------------------------------------------------------
  
  From <- NULL
  To <- NULL

  # calcul des positions de start dans la sequence
  for(i in 1:length(start)){
    new <- exactWord(seqno=seqno, word=start[i], from=phase, step=3)[[1]]
   if(is.null(new)){
    return(NULL) }
  else From <- c(From,new)  
  }
  
 # calcul des positions de stop dans la sequence
  for(i in 1:length(stop)){
    new<-exactWord(seqno=seqno, word=stop[i], from=phase, step=3)[[1]]
  if(is.null(new)){
    return(NULL) }
  else To <- c(To,new)  
  }

  long <- sizeSeq(seqno)+getBegSeq(seqno)-1
  longreal <- sizeSeq(seqno)
  endPhase <- ((longreal-phase) %/% 3)*3 + getBegSeq(seqno) + phase -1
  From <- sort(From)
  To <- sort(To)

  ## enlever tous les -1 (aucun codon start parmis les "start" dans la squence qui soit dans la phase). La boucle est necessaire car il peut y avoir plusieurs -1 correspondant au nombre de codons start donns.
  while((From[1]== -1) & (length(From) > 0) )
    From <- From[-1]
  while((To[1]== -1) & (length(To) > 0) )
    To <- To[-1]



  # rajouter l'orf incomplet cot 5' dont le stop est le plus petit et le start correspond  la phase

  st<-phase
  sp <- min(c(To,(endPhase-2))+2)
   
  From <- c(phase,From)


  # rajouter les orfs incomplettes cot 3'
  maxStop <- max(c(To,0))

  From2 <- From[From>maxStop]
  To2<- rep(endPhase,length(From2))
  
  res <- cbind(c(st,From2),c(sp,To2)) 
  if(suborfs)
    return(res)
  else
    return(unionSeg(res))
  
}

##########################################################################################

getOrfs <- function (phase=NULL, seqno=0,
                     start="atg",stop=c("taa","tag","tga"),
                     complete=TRUE, suborfs=TRUE)
{
#------------------------------------------------------------------------------------
# getOrfs
#
# Description :
#
# Gets complete or all Orfs in a sequence for a given phase
#
# Usage :
#
# getOrfs (phase=NULL,seqno=0,start="atg",stop=c("taa","tag","tga"), complete=TRUE)
#
# Arguments :
#
# phase      integer/scalar, 1,2 or 3, NULL for all three phases
# seqno      integer/scalar, Sequence number (buffer number)
# start      string/vector, start codons
# stop       string/vector, stop codons
# complete   logical/scalar, if complete=TRUE, results are only complete Orfs, else all Orfs 
#
# Value :
#
# getOrfs returns a table of positions
#
# Call:
#
# .allOrfs1phase, .allOrfs1phaseComplete
#  
# Author
#
# Emna
# 24 Nov 2005
#-------------------------------------------------------------------------------------
  lestrand <- getStrand()
  setStrand(0) ## negative phase: not implemented
 
  all2 <- NULL

  if(is.null(phase))
    all1 <- rbind(.allOrfs1phase(phase=1,seqno=seqno,start=start,
                                 stop=stop, suborfs=suborfs),
                  .allOrfs1phase(phase=2,seqno=seqno,start=start,
                                 stop=stop, suborfs=suborfs),
                  .allOrfs1phase(phase=3,seqno=seqno,start=start, stop=stop, suborfs=suborfs))
  else
    all1 <- .allOrfs1phase(phase=phase,seqno=seqno,start=start,stop=stop, suborfs=suborfs)  
  
  if (complete==FALSE){
    if(is.null(phase)){
      all2 <- rbind(.allOrfs1phaseComplete(phase=1,seqno=seqno,start=start,
                                           stop=stop,suborfs=suborfs),
                    .allOrfs1phaseComplete(phase=2,seqno=seqno,start=start,
                                           stop=stop,suborfs=suborfs),
                    .allOrfs1phaseComplete(phase=3,seqno=seqno,start=start,
                                           stop=stop,suborfs=suborfs))
    }
     
    else{
      all2 <-
      .allOrfs1phaseComplete(phase=phase,seqno=seqno,start=start,
      stop=stop,suborfs=suborfs)
    }
  }
  all <- rbind(all2,all1)     
   

  setStrand(lestrand)

  if(length(all)>2)
      return(all[order(all[,1]),])

  all
}




#
# Find largest ORF
#
maxOrf <- function(seqno=0,phase=NULL,start="atg",
                   stop=c("taa","tag","tga"),complete=TRUE)
{
  
  all <-  getOrfs(phase=phase,seqno,start=start,stop=stop,complete=complete)
  
  
  if(length(all)>0)
    {
      if(is.na(all[1])>0)
        return(NA)
     max(all[,2]-all[,1]+1)
    }
  else
    -1
}

