# ----------------------------------------------------
# @File: readlocation.R
# @Synopsis: functions getting locations from Embl / Gbk file
#
# @History: <Gloup> 10 Feb 2005 created by A. LUCAS
# Last modified: Time-stamp: <2006-01-31 14:02:45 lucas>
#
# ----------------------------------------------------



# ==============================================================================
# @Function: lib.feat.read.embl.location 
# @Type: C
# @Purpose:  Extract all locations of "key" in an EMBL entry
# @Args:  name  : entry name, format "BankFileName:ACNumber"
# @+	  beg   : debut de la region d'interet
# @+	  end   : fin de la region d'interet
# @+	  key   : mot cl dirigeant la recherche
# @+      subkey: second partern to find into first key
# @Returns      : Une liste compose de 
# @+      locations.begins : Positions de debut correspondant au mot cl key
# @+	  locations.ends   : Positions de fin correspondant au mot cl key
# @+	  locations.begins.limits : Pour chaque position de dbut, indique si elle est exacte(0) ou floue (1)
# @+	  locations.ends.limits   : Pour chaque position de fin, indique si elle est exacte(0) ou floue (1) 	
# @+	  locations.strand        : Pour chaque position, indique si elle est dans le sens 5'-3' (0) ou dans le sens 3'-5'	   
# @Uses:   Read_EMBL_Location_R(C), abi.sys.xtoffsetseq,abi.sys.sizeseqembl
# =============================================================================



readEmblLocation  <- function(file,name=NA,
                                from=1,to=0,
                                key="CDS",subkey="")

{

  indexpath <- paste(file,".ix",sep="")
  if (makeIndex (file = file, type = "E")  == -1)
    {
      warning("error in index built")
      return (NULL)
    }

  if ((file.exists (file) == FALSE))
    {
      warning("file doesn't exist")
      return(NULL)
    }

  if(is.na(name))
    ## fichier avec une seule sequence ou pas de fichier banks dans bk
    ## On prend la premiere ligne du fichier index pour prendre l'accn
    name <- as.character(read.table(indexpath,nrows=2)[1,1])

  param <- .paramFromIndex (name, indexpath)
  if (param[5] == -1)
    {
      print ("<readseq paramFromIndex error>")
      return(-1)
    }
  
    
  if (to == 0 )
    ##    to <- .C("sizeseqemblC", name, param[[4]], file,PACKAGE="GeneR")[[2]]
    to <- param$size

  res <- .Call("readLocation",as.character(file),
               as.integer(param[[3]]),
               as.integer(from),
               as.integer(to),
               as.character(key),
               as.character(subkey),
               "SQ",
               PACKAGE="GeneR")

  if(is.list(res))
   {
     if(length(res) == 7)   
       return(data.frame(from=res[[4]],to=res[[5]],subkey=res[[2]],strand=res[[3]],fuzzystart=res[[6]],fuzzyend=res[[7]],locus=res[[1]]))
     else
       return(data.frame(from=res[[3]],to=res[[4]],strand=res[[2]],fuzzystart=res[[5]],fuzzyend=res[[6]],locus=res[[1]]))
   }
  
  

  if(res[1] == 0)
    return(NULL)

  return(res)
  
}







readGbkLocation  <- function(file,name=NA,
                                from=1,to=0,
                                key="CDS",subkey="")

{

  indexpath <- paste(file,".ix",sep="")
  if (makeIndex (file = file, type = "G")  == -1)
    {
      warning("error in index built")
      return (NULL)
    }

  if ((file.exists (file) == FALSE))
    {
      warning("file doesn't exist")
      return(NULL)
    }

  if(is.na(name))
    ## fichier avec une seule sequence ou pas de fichier banks dans bk
    ## On prend la premiere ligne du fichier index pour prendre l'accn
    name <- as.character(read.table(indexpath,nrows=2)[1,1])

  param <- .paramFromIndex (name, indexpath)
  if (param[5] == -1)
    {
      print ("<readseq paramFromIndex error>")
      return(-1)
    }
  
    
  if (to == 0 )
    to <- param$size

  res <- .Call("readLocation",as.character(file),
               as.integer(param[[3]]),
               as.integer(from),
               as.integer(to),
               as.character(key),
               as.character(subkey),
               "OR",
               PACKAGE="GeneR")

  if(is.list(res))
    {
     if(length(res) == 7)   
       return(data.frame(from=res[[4]],to=res[[5]],subkey=res[[2]],strand=res[[3]],fuzzystart=res[[6]],fuzzyend=res[[7]],locus=res[[1]]))
     else
       return(data.frame(from=res[[3]],to=res[[4]],strand=res[[2]],fuzzystart=res[[5]],fuzzyend=res[[6]],locus=res[[1]]))
    }
 
  if(res[1] == 0)
    return(NULL)
  
  return(res)
  
}


readEmblDescript <- function(file,name=NA,code="DE")
  {
  indexpath <- paste(file,".ix",sep="")
  if (makeIndex (file = file, type = "E")  == -1)
    {
      warning("error in index built")
      return (NULL)
    }

  if ((file.exists (file) == FALSE))
    {
      warning("file doesn't exist")
      return(NULL)
    }

  if(is.na(name))
    ## fichier avec une seule sequence ou pas de fichier banks dans bk
    ## On prend la premiere ligne du fichier index pour prendre l'accn
    name <- as.character(read.table(indexpath,nrows=2)[1,1])

  param <- .paramFromIndex(name,indexpath)

    .Call("readEmblDescript",
          as.character(file),
          as.integer(param[3]),
          as.integer(param[4]),
          as.character(code),
          PACKAGE="GeneR")


  }
