## ----------------------------------------------------
## @File: readseq.R
## @Synopsis: All functions related to read anything
##            concerning Embl, Gbk & Fasta files
##            (except 3 functions strReadEmbl/gbk/fasta).
##
## @History: <Gloup> 10 Feb 2005 created by A. LUCAS
## Last modified: Time-stamp: <2006-01-10 17:33:43 lucas>
##
## ----------------------------------------------------




## --------------------
## @Function: abi.sys.readseqgbk (namebk,offsq,begseq=1,endseq=0,seqno=0)
## @+	     readseqgbk
## @Type: C
## @Purpose:  acces direct  un extrait de sequence dans
## @+         une banque GenBank namebk.
## @+         Installe la sequence dans le buffer seqno
## @+         Si endseq = 0,lit toute la sequence
## @Note:     < LC > add Nov 01
## --------------------

.readseqgbk <- function(namebk,offsq,begseq=1,
                                             endseq=0,seqno=0,upper=1,err=0)
{
	
	if (begseq == 0)
		begseq <-1
		
	.C("ReadSeqGBKR",
           as.character(namebk),
           as.integer(offsq),
           as.integer(begseq),
           as.integer(endseq),
           as.integer(seqno),
           as.integer(upper),
           as.integer(err),
           PACKAGE="GeneR") [[5]]
}
## --------------------
## @Function: abi.sys.readseqembl (namebk,offsq,begseq=1,endseq=0,seqno=0)
## @+	     readseqembl
## @Type: C
## @Purpose:  acces direct  un extrait de sequence dans
## @+         une banque namebk. Installe la sequence dans le buffer seqno
## @+         Si endseq = 0,lit toute la sequence
## @Note:     < LC > add Feb 00
## --------------------

.readseqembl  <- function(namebk,offsq,begseq=1,
						endseq=0,seqno=0,upper=1,err=0)
{
	
	if (begseq == 0)
		begseq <-1
		
	.C("ReadSeqEMBLR",
           as.character(namebk),as.integer(offsq),
           as.integer(begseq),as.integer(endseq),
           as.integer(seqno),as.integer(upper),
           as.integer(err),
           PACKAGE="GeneR") [[5]]
}

## --------------------
## @Function: .readseqfasta (namebk,offsq,begseq=1,endseq=0,seqno=0)
## @+ 	     readseqfasta
## @Type: C
## @Purpose:  acces direct  un extrait de sequence dans
## @+         une banque namebk. Installe la sequence dans le buffer seqno
## @+         Si endseq = 0,lit toute la sequence
## @Note:     < LC > add Feb 00
## --------------------

.readseqfasta <- function(namebk,offsq,begseq=1,
                         endseq=0,seqno=0,upper,err=0)
{
	if (begseq == 0)
		begseq <-1
	.C("ReadSeqFASTAR",as.character(namebk),as.integer(offsq),
           as.integer(begseq),as.integer(endseq),
           as.integer(seqno),as.integer(upper),as.integer(err),
           PACKAGE="GeneR") [[5]]
}



## --------------------
## @Function:readSeq (file,name = NA,path = "", type = "E", seqno = 0,
##                    from = 1, to = 0, index = "ix")
## @+	     readSeq
## @Type: R
## @Purpose:  acces direct  un extrait de sequence dans
## @+         une banque namebk. Installe la sequence   
## @+	     dans le buffer seqno			
## @+         Si endseq = 0,lit toute la sequence	
## @Note:    < LC > created Feb 00
## --------------------

readSeq  <- function (file,name = NA, type = "F", seqno = 0,
                      from = 1, to = 0, upper=TRUE,index = "ix")
{
  freeSeq(seqno)

  indexpath <- paste (file, index, sep=".")
  
  if (makeIndex (file = file, type = type, index = index) == -1)
    {
      warning ("error in index built")
      return (-1)
    }
  
  if ((file.exists (file) == FALSE) || (file.exists (indexpath) == FALSE))
    {
      warning ("the bank file or the index file doesn't exist")
      return (-1)
    }

  
  if (is.na (name))
    ## fichier avec une seule sequence ou pas de fichier banks dans bk    
    ## On prend la premiere ligne du fichier index pour prendre l'accn
    name <- as.character(read.table(indexpath,nrows=2)[1,1])
    
  param <- .paramFromIndex (name, indexpath)
  if (param[[5]] <= 0 )
    {
      print ("<readseq paramFromIndex error>")
      return(-1)
    }
     
  if (as.character (type) == "F")
    sizeSeq <- .C ("sizeseqfasta", name, param[[4]], file,PACKAGE="GeneR") [[2]]
  if (as.character (type) == "E")
    sizeSeq <- .C ("sizeseqemblC", name, param[[4]], file,PACKAGE="GeneR") [[2]]
  if (as.character (type) == "G")
    sizeSeq <- .C ("sizeseqgbk", name, param[[4]], file,PACKAGE="GeneR") [[2]]

  if (sizeSeq == -1)
    {
      warning ("readseq sizeSeq error")
      return (-1)
    }

  if (to == 0)
    to <- sizeSeq
  offsetseq <- param[[4]]
  if (to > sizeSeq)
    {
      warning ("to larger than sizeSeq")
      return (-1)
    }
  if (from < 1)
    {
      print ("from < 1")
      return (-1)
    }


  err <- 0
  ##print  ("Recuperation de la sequence")
  if (type == "E")
    .readseqembl (file, offsetseq, from, to, seqno, upper,err)
  if (type == "F")
    .readseqfasta (file, offsetseq, from, to, seqno,upper, err)
  if (type == "G")
    .readseqgbk (file, offsetseq, from, to, seqno, upper,err)
  
  if (err != -1)
    {
      setSeqSize (pos = sizeSeq, seqno = seqno)
      setBegSeq (pos = from, seqno = seqno)
      setAccn (name = name, seqno = seqno)
    }
  return (err)		
}




# ------------------------------------------------------- 
# @Function: abi.sys.sizeseqembl (name,path=".")	
# @+	     sizeseqembl	   
# @Purpose:  retourne la longueur de la sequence dont 	   
# @+         l'accn est rpertori dans indexpath		   	
# @Note:     < LC > add 25 May 00			   	
# ------------------------------------------------------- 

sizeSeqEmbl <- function(file,name=NA, index = "ix")
{

  indexpath <- paste(file, index, sep=".")
  err <- 0
  if(makeIndex(file=file, type="E", index=index) == -1)
    {
      print("Error in index built")
      return (NULL)
    }
 
  if ((file.exists (file) == FALSE) || (file.exists (indexpath) == FALSE))
    {
      print("the bank file or the index file doesn't exist")
      print (paste ("bank:", file, "index:", indexpath))
      return(NULL)
    }
  
  if(is.na(name))
    ## fichier avec une seule sequence ou pas de fichier banks dans bk        
    ## On prend la premiere ligne du fichier index pour prendre l'accn
    name <- as.character(read.table(indexpath,nrows=2)[1,1])
    
  param <- .paramFromIndex (name,indexpath)
  
  if (param[5] == -1)
    {
      warning("<readseq paramFromIndex error>")
      return(NULL)
    }
      
  err <- .C("sizeseqemblC",as.character(name), param[[4]], as.character(file),
            PACKAGE="GeneR")[[2]]

  if (err == -1)
    {
      warning("sizeseqembl ERROR")
      return(NULL)
    }
  
  err	
  
}

# ------------------------------------------------------- 
# @Function: abi.sys.sizeseqgbank (name,path=".")	
# @+	     sizeseqgbank	   
# @Purpose:  retourne la longueur de la sequence dont 	   
# @+         l'accn est rpertori dans indexpath		   	
# @Note:     < LC > add 25 May 00			   	
# ------------------------------------------------------- 

sizeSeqGbk  <- function(file,name=NA)
{
  index <- "ix"
  indexpath <- paste(file, index, sep=".")

  if(makeIndex(file=file, type="G", index=index) == -1)
    {
      print("Error in index built")
      return (NULL)
    }
  if ((file.exists (file) == FALSE) || (file.exists (indexpath) == FALSE))
    {
      print("the bank file or the index file doesn't exist")
      print (paste ("bank:", file, "index:", indexpath))
      return(NULL)
    }
  
  if(is.na(name))
    ## fichier avec une seule sequence ou pas de fichier banks dans bk
    ## On prend la premiere ligne du fichier index pour prendre l'name
    name <- as.character(read.table(indexpath,nrows=2)[1,1])
        
  param <- .paramFromIndex (name,indexpath)
      
  if (param[5] == -1)
    {
      warning("<readseq paramFromIndex error>")
      return(NULL)
    }

  err <- .C("sizeseqgbk", name, param[[4]], file,PACKAGE="GeneR")[[2]]

  if (err == -1)
    {
      warning("sizeseqgbk ERROR")
      return(NULL)
    }

  err	
}


# --------------------
# @Function: abi.sys.sizeseqfasta (name,path="." )
# @+         sizeseqfasta
# @Type: C
# @Purpose:  donne la taille d'une sequence fasta repertorie dans le fichier indexpath
# @Note:    < LC > created  11 May 00
# --------------------

sizeSeqFasta <-  function (file,name=NA)
{
  index <- "ix"
  indexpath <- paste(file, index, sep=".")
  err <- 0
  if(makeIndex(file=file, type="F", index=index) == -1)
    {
      print("Error in index built")
      return (NULL)
    }
  if ((file.exists (file) == FALSE) || (file.exists (indexpath) == FALSE))
    {
      print("the bank file or the index file doesn't exist")
      print (paste ("bank:", file, "index:", indexpath))
      return(NULL)
    }
  
  if(is.na(name))
    ## fichier avec une seule sequence ou pas de fichier banks dans bk
    ## On prend la premiere ligne du fichier index pour prendre l'accn
    name <- as.character(read.table(indexpath,nrows=2)[1,1])
      
  param <- .paramFromIndex (name,indexpath)
      
  if (param[5] == -1)
    {
      print("<readseq paramFromIndex error>")
      return(NULL)
    }

  err<-.C("sizeseqfasta", as.character(name),as.integer(param[[4]])
          ,as.character(file),PACKAGE="GeneR") [[2]]

  if (err == -1)
    {
      warning("sizeseqfasta ERROR")
      return(NULL)
     } 
  err	

}


# Get a feature from a Fasta file
fastaDescription<- function(file,name="")
{

  makeIndex(file,type="F")
  if(name=="")
    name <- as.character(read.table(paste(file,"ix",sep='.'),nrows=2)[1, 1])
  xt <- .paramFromIndex(name,paste(file,"ix",sep='.'))

  if(xt[[5]] == -1)
    {
      print("Name not in file of problem in index file")
      return(-1)
    }
  
  .Call("fasta_descript",
        as.character(file),
        as.character(name),
        as.integer(xt[[3]]),
        as.integer(xt[[4]]),
        PACKAGE="GeneR"
        )

}
    



# Simple aliases... for lazy...

readFasta <- function(file,name=NA, seqno=0,
                      from=1, to=0, upper = TRUE,index="ix")
  {
    readSeq(file=file,name=name, type="F", seqno=seqno, from=from, to=to, index=index,upper = upper)
  }
readGbk <- function(file,name=NA, seqno=0,
                      from=1, to=0, upper = TRUE, index="ix")
  {
    readSeq(file=file,name=name, type="G", seqno=seqno, from=from, to=to, index=index,upper=upper)
  }

readEmbl <- function(file,name=NA, seqno=0,from=1, to=0, upper = TRUE, index="ix")
  {
    readSeq(file=file,name=name, type="E", seqno=seqno, from=from, to=to, index=index,upper=upper)
  }

