

getSeq  <-  function (seqno=0,strand = getStrand(), from = 1, to = 0 )
{
  lestrand <- getStrand()
  if(sizeSeq(seqno) == 0)
    {
      warning("No sequences in buffer seqno")
      return(NULL)
    }
  if(length(from) != length(to))
    {
      warning("from and to must be of same length")
      return(NULL)
    }


  limites <- list(from,to)

  setStrand(strand)
  liste <- .transf.limites(seqno, limites)
	
  if(liste[[1]] == -1)
    {
      warning("error in the transformation of the limits")
      return (NULL)
    }
  
  from <- liste[[2]]
  to <- liste[[3]]

  res <- .Call("multiextract",as.integer(seqno),
               as.integer(from),
               as.integer(to),
               as.integer(strand),
               PACKAGE="GeneR") 
  
  if (res[[1]] == -1)
    return(NULL)

   #print ("multi extract")
  setStrand(lestrand)
  
  return (res)
}



# ---------------------------------------------------- 
# @Function: abi.sys.assemble (seqno, from, to, bufdest)
# @+	     assemble 		
# @Purpose:  cre une nouvelle squence en concatnant les rgions
# @+	     prcises par from et to de la squence contenue dans seqno
# Note:	     [LC] created Dec 00		
# ----------------------------------------------------

assemble <- function(seqno=0, from = 1,to = 0,strand=getStrand(), 
					destSeqno = 1)
{

  lestrand <- getStrand()
  freeSeq(destSeqno)
  if(sizeSeq(seqno) == 0)
    {
      warning("No sequences in buffer seqno")
      return(-1)
    }

  if(length(from) != length(to))
    {
      warning("from and to must be of same length")
      return(-1)
    }

  if(length(strand) == 1)
    strand <- rep(strand,length(from))

  if(sum(strand == 1)>0)
    if (revComp(seqno) == -1)
      return (-1)
	
  from[from==0] <- getEndSeq(seqno)
  to[to==0] <- getEndSeq(seqno)

  vecL <- to-from+1

  l <- sum(vecL)

  .seqalloc (l+1, seqno = destSeqno, fill = 1)

  tab <- array(c(from,to, strand), dim=c(length(from),3))

  lst <- unlist(apply(tab,1,list),recursive = FALSE)

  lapply(lst,.abi.sys.ass,seqno=seqno, bufdest=destSeqno)

  setStrand(lestrand)

  ## now made direcly by .seqalloc
  ##  setBegSeq(1, destSeqno)
  ##  setEndSeq(l, destSeqno)
  ##  setSeqSize(l,destSeqno)
  
  ##  setAccn("Seq_R", destSeqno)
  
  
  destSeqno
  
}


# -------------------------------------------------------------------
# ass : fonction appele par assemble
# -------------------------------------------------------------------


.abi.sys.ass <- function(lst,seqno=0,bufdest = 1)
{

  from <- lst[[1]]
  to <- lst[[2]]
  strand <- lst[[3]]

  setStrand(strand)

  lims <- .transf.limites(seqno = seqno,list(from,to))
  
  from <- lims[[2]]
  to <- lims[[3]]
  if(lims[[1]] == -1)
    return(-1)

  
  res<-.C("assemble", as.integer(seqno), as.integer(from), as.integer(to), 
          as.integer(bufdest), as.integer(strand), err=as.integer(-1),
          PACKAGE="GeneR")[[6]]
  if(!res) 
    {
      warning("Problem with assemble")
      return(-1)   
    }
  res		 
}


# --------------------
# @Function: abi.sys.concat(seq.one=0, seq.two=1, seq.dest=2,
# @+		from1=1,to1=0,strand1=0, from2=1,to2=0,strand2=0)
# @+	     concat
# @Type: C
# @Purpose:  Concatene 2 sequences dans une troisieme
# @Note:    < LC > created Dec 00
# --------------------

concat <- function(seqno1=0, seqno2=1, destSeqno=2,
                   from1=1,to1=0,strand1=getStrand(), from2=1,to2=0,strand2=getStrand())
{
  lestrandla <- getStrand()

  freeSeq(destSeqno)
  if(sizeSeq(seqno1) == 0 || sizeSeq(seqno2) == 0)
    return(-1)

  setStrand(strand1)

  limites1 <- list(from1,to1)	
  liste <- .transf.limites(seqno=seqno1, limites1)
  if(liste[[1]] == -1)
    {
      warning("error in the transformation of the limits")
      return (NULL)
    }		
  from1 <- liste[[2]]
  to1 <- liste[[3]]




	
  setStrand(strand2)
  limites2 <- list(from2,to2)

  liste <- .transf.limites(seqno=seqno2, limites2)
  if(liste[[1]] == -1)
    {
      warning("error in the transformation of the limits")
      return (NULL)
    }		
  from2 <- liste[[2]]
  to2 <- liste[[3]]

  if (all(from1<=to1)==FALSE || all(from2<=to2)==FALSE)
    {
      warning("error: from must be <= to")
      return(-1)
    }

  if((all(from1 <= sizeSeq(seqno1))==FALSE) ||
     (all(to1 <= sizeSeq(seqno1))==FALSE) ||
     (all(from1 != -1)==FALSE) || (all(to1!=-1)==FALSE) ||
     (all(from2 <= sizeSeq(seqno2))==FALSE) ||
     (all(to2 <= sizeSeq(seqno2))==FALSE) ||
     (all(from2 != -1)==FALSE) || (all(to2!=-1)==FALSE) )
    {
      warning("error with from and to")
      return(-1)
    }


  res <- .C("sys_concat", as.integer(seqno1), as.integer(seqno2),
     as.integer(destSeqno),as.integer(from1),
     as.integer(to1), as.integer(strand1),
     as.integer(from2),as.integer(to2),
     as.integer(strand2),PACKAGE="GeneR") [[3]]

##  setEndSeq(pos=sizeSeq(seqno=destSeqno),seqno=destSeqno)
  setBegSeq(pos=1,seqno=destSeqno)
  setAccn("concat",seqno=destSeqno)
  setStrand(lestrandla)
  res
}




appendSeq <- function(destSeqno=0, seqno=1)
  {
    
    res <- .C("concat",
              as.integer(destSeqno),
              as.integer(seqno),
              err=as.integer(0),
              PACKAGE="GeneR")
    if(res$err == -1)
      return(NULL)
    if(res$err == -2)
      {
        warning("Memory allocation error")
        return(NULL)
      }
    return(0)
  }




# --------------------
# @Function: abi.sys.placeString (s, seqno = 0)
# @+	     placestring
# @Type: C
# @Purpose: place une sequence dans un buffer
# @+        returns seqno
# @Note: < LC > created Feb 00
# --------------------

placeString  <- function(s,seqno = 0,upper=TRUE)
{
  freeSeq(seqno)
  err <- .C("sys_placestring", as.character(s) ,
            as.integer(seqno),as.integer(upper),PACKAGE="GeneR") [[2]]



  return (err)
}

# @Function: abi.str.insert (s, insert, from = 0)
# @Type: R
# @Purpose: insert 'insert' at position 'from' in 'seqno'
# @+        if from == 0 insert at end
# @+        if from < 0 count 'from' from the end of 's'
# @+        returns the created string
# --------------------

insertSeq <- function(s, insert, from = 0)
{
    ls <- nchar(s)

    if (from <= 0)
       from <- ls + from + 1

#    beg <- ifelse(from > 1,   .abi.str.substr(s, 1, from-1), "")
        beg <- ifelse(from > 1,  substring(s,1,from-1) , "")
    end <- ifelse(from <= ls, substring(s, from, ls), "")

    paste(beg, insert, end, sep="")
}
