## ----------------------------------------------------
## @File: strreadseq.R
## @Synopsis: All functions related to read sequences
##            from Embl, Gbk & Fasta files to a character
##            string.
##
## @History: <Gloup> 9 Feb 2005 created by A. LUCAS
## Last modified: Time-stamp: <2005-11-03 17:10:57 lucas>
##
## ----------------------------------------------------

strReadEmbl <- function (file, name = NA, from = 1,
                         to = 0, upper=TRUE,index = "ix") 
{
  indexpath <- paste(file,".",index,sep="")

  if(makeIndex(file=file, type="E") == -1)
    {
      print("Error in index built")
      return (NULL)
    }

  if(is.na(name))
    name <- as.character(read.table(indexpath,nrows=2)[1,1])

  ## Get some parameters from index file
  param <-  .paramFromIndex(accn=name,indexpath=indexpath)

  sizeSeq <- .C ("sizeseqemblC", accn=name,
                 param[[4]], file,PACKAGE="GeneR") [[2]]
  
  if((to == 0) || (to > sizeSeq) )
    to <- sizeSeq
  sizeread <- as.integer( to - from + 2)
  offset <- as.integer(param$beginSeq)
  
  ## Allocation
  seq <- .Call("alloc_char",sizeread,as.integer(1),PACKAGE="GeneR")

  list <- .C("ReadSeqEMBL",seq=seq,file=file,offsq=offset,
             begseq=as.integer(from),
             endseq=as.integer(to),
             as.integer(upper),
             as.integer(0),
             PACKAGE="GeneR")
  list$seq
}
  
strReadGbk <- function (file, name = NA, from = 1,
                         to = 0,upper=TRUE) 
{
  indexpath <- paste(file,".ix",sep="")

  if(makeIndex(file=file, type="G") == -1)
    {
      print("Error in index built")
      return (NULL)
    }

  if(is.na(name))
    name <- as.character(read.table(indexpath,nrows=2)[1,1])

  ## Get some parameters from index file
  param <-  .paramFromIndex(accn=name,indexpath=indexpath)

  sizeSeq <- .C ("sizeseqgbk", accn=name,
                 param[[4]], file,PACKAGE="GeneR") [[2]]
  
  if((to == 0) || (to > sizeSeq) )
    to <- sizeSeq
  sizeread <- as.integer( to - from + 2)
  offset <- as.integer(param$beginSeq)
  
  ## Allocation
  seq <- .Call("alloc_char",sizeread,as.integer(1),PACKAGE="GeneR")

  list <- .C("ReadSeqGBK",seq=seq,file=file,offsq=offset,
             begseq=as.integer(from),
             endseq=as.integer(to),
             as.integer(upper),
             as.integer(0),
             PACKAGE="GeneR")
  list$seq
}
  
strReadFasta <- function (file, name = NA, from = 1,
                         to = 0,upper=TRUE) 
{
  indexpath <- paste(file,".ix",sep="")

  if(makeIndex(file=file, type="F") == -1)
    {
      print("Error in index built")
      return (NULL)
    }

  if(is.na(name))
    name <- as.character(read.table(indexpath,nrows=2)[1,1])

  ## Get some parameters from index file
  param <-  .paramFromIndex(accn=name,indexpath=indexpath)

  sizeSeq <- .C ("sizeseqfasta", accn=name,
                 as.integer(param[[4]]),
                 file,PACKAGE="GeneR") [[2]]

  
  if((to == 0) || (to > sizeSeq) )
    to <- sizeSeq
  sizeread <- to - from + 1
  offset <- as.integer(param$beginSeq)
  
  ## Allocation
  seq <- .Call("alloc_char",as.integer(sizeread),as.integer(1),PACKAGE="GeneR")

  list <- .C("ReadSeqFASTA",seq=seq,file=file,
             offsq=as.integer(offset),
             begseq=as.integer(from),
             endseq=as.integer(to),
             as.integer(upper),
             err=as.integer(0),
             PACKAGE="GeneR")
  if(list$err)
    return(list$seq)
  else
    NULL
}
  
