# ----------------------------------------------------
# @File: webutils.R
# @Synopsis: functions getting informations on web
#
# @History: <Gloup> 30 Jun 2004 created by A. LUCAS
# Last modified: Time-stamp: <2005-11-21 11:30:26 lucas>
#
# ----------------------------------------------------




# ==============================================================================
# @Function: sequrl
# @Type: R
# @Purpose:  Get trough the web a seqence in Fasta / Gbk format
# @Args:  accno      : entry name, format ">dbj|BY608190.1|BY608190"
# @+                   or "BY608190"
# @+      sousmotif  : TRUE look for BY608190 instead of
# @+                   >dbj|BY608190.1|BY608190
# @+	  srs        : url of a srs web program
# @+	  type       : fasta or genbank
# @Returns           : A file
# =============================================================================

seqSrs <- function (accno,file="toto.seq",submotif=FALSE,srs="http://srs.ebi.ac.uk/srsbin/cgi-bin/wgetz",bank=c("EMBL","REFSEQ"))
  {
    # This function will get trough a srs program a sequence to
    # the file with accnumber as input 

    

      deb <- "?-ascii+-vn+2+["  

    
    if (submotif)
      {
        char <- accno
        #char <- substring(accno,2)
        char <- strsplit(as.character(char),'\\|')[[1]][2]
        char <- strsplit(as.character(char),'\\.')[[1]][1]
        if(!is.na(char))
          accno <- char
      }

    bank <- paste(bank,"-AccNumber:",accno,sep="")

    lesbank <- bank[1]
    if(length(bank)>1)
      for(i in 2:length(bank))
        {   
          lesbank <- paste(lesbank,bank[i],sep=']|[')
        }
    
    urldemande <- paste(srs,deb,lesbank,"]",sep="")
    download.file(urldemande,destfile=file)


      lines <- readLines(file)
      ## Verif du resultat
      if(length(lines) < 3)
        {
          warning("Download Pb (1)")
          return(NULL)
        }
      if(length(grep("SRS error",lines[3])))
        {
          warning("Srs Error")
          return(NULL)
        }
      ## On regarde les lignes qui contiennent "//" 
      sup <- grep("^//",lines)
      write.table(file=file,lines[1:(sup[1])],quote=FALSE, row.names=FALSE, col.names =FALSE)

      1
    }



seqNcbi <- function (accno,file="toto.seq",submotif=FALSE,type="fasta")
  {
    # This function will get trough a ncbi (web) a sequence to
    # the file with accnumber as input 

    if(length(grep(type,c('gbk','genebank','genbank'), ignore.case=TRUE)))   
      type<-"gbwithparts"
    else
      type <- "fasta"
    
    if (submotif)
      {
        char <- accno
        #char <- substring(accno,2)
        char <- strsplit(as.character(char),'\\|')[[1]][2]
        char <- strsplit(as.character(char),'\\.')[[1]][1]
        if(!is.na(char))
          accno <- char
      }
    
    urlbase <- "http://www.ncbi.nlm.nih.gov/entrez/viewer.fcgi?"
    #parmsbase <- "cmd=&txt=on&save=&cfm=&extrafeat=16&term=&dispmax=20&SendTo=t&__from=&__to=&__strand=&db=nucleotide&view="
    parmsbase <- "db=nucleotide&qty=1&c_start=1&send=Send&sendto=t&from=begin&to=end&extrafeatpresent=1&ef_MGC=16&dopt="
    urldemande <- paste(urlbase,parmsbase,type,"&list_uids=",accno,sep="")
    download.file(urldemande,destfile=file)
 
    if (type== "genebank") 
      {
        lines <- readLines(file)
        # Verif du resultat
        if(length(lines) < 3)
          {
            warning("Download Pb (1)")
            return(NULL)
          }
        if(length(grep("ERROR",lines[3])))
          {
            warning("NCBI Error")
            return(NULL)          
          }
      }
    else
      {
        lines <- readLines(file)
        
        # Verif du resultat
        if(length(lines) < 2)
          {
            warning("Download Pb (1)")
            return(NULL)
          }
        if(length(grep("ERROR",lines[3])))
           {
            warning("NCBI Error")
            return(NULL)          
          }
        write.table(file=file,lines[lines != ""],quote=FALSE, row.names=FALSE, col.names =FALSE)
        
      }
    1
  }

