## ----------------------------------------------------
## @File: writeFasta.R
## @Synopsis: write fasta bank file
##
## @History: <Gloup> 02 nov 2005 created by A. LUCAS from L. Cotteret codes
## 
## 
## Last modified: Time-stamp: <2005-11-21 16:08:08 lucas>
##




## ---------------------------------------------------- 
# @Function: abi.sys.write.fasta.seq (seqno, from, to, name, comment, file)
# @+	     write.fasta.seq 		
# @Purpose:  ecrit une  portion de la sequence 
# @+	     en format fasta dans un fichier
# Note:	     [LC] created 22 Oct 00		
# ---------------------------------------------------- 
writeFasta <- function( file="data.fasta",
                        seqno=0,
                        from=getBegSeq(seqno),
                        to=getEndSeq(seqno),
                        name=getAccn(seqno),
                        comment=paste("from",from,"to",to),                   
                        cpl=60,append = FALSE)
  {

    if(name[1]=="")
      name <- "Seq_R"

    if(name == getAccn(seqno))
      if( (from!=1) || (to!=getEndSeq(seqno)))
        name <- paste(name,from,to,sep="_")
    
    limites <- list(from,to)
	
    liste <- .transf.limites(seqno, limites)
	
    if(liste[[1]] == -1)
      {
	warning("error in the transformation of the limits")
	return (-1)
      }
		
    nfrom <- liste[[2]]
    nto <- liste[[3]]
	
    comp <- getStrand()

    if (comp == 1)
      {
	if (revComp(seqno) == -1)
          return (-1)
		
        if(comment[1] == paste("from",from,"to",to)[1])
          comment <- paste("complementaire", comment, sep = " ");
      }

    if (append == FALSE)
      file.remove(file)


    if((length(name) < length(from)) || (length(comment) < length(from)))
        {
          warning("Name or Comment have not the same size of from")
          return(-1)
        }
        
    .C("sys_write_fasta_seq", as.integer(seqno), as.integer(nfrom),
       as.integer(nto), as.character(name),
       as.character(comment), as.character(file),
       as.integer(comp), as.integer(cpl),
       as.integer(length(nfrom)),
       err=as.integer(1),
       PACKAGE="GeneR")$err
								
    

  }


strWriteFasta <- function (s,
                           file="data.fasta",                           
                           name="Seq_R",
                           comment=as.character(nchar(s)),
                           cpl=60,append = FALSE)
{
  if(is.null(s))
    return(NULL)
  if(nchar(s) == 0)
    return(NULL)
  
  s <- as.character(s)
  n <- nchar(s)
  .C("write_fasta_seq", s, as.integer(1),
     as.integer(n), as.character(name),
     as.character(comment), as.character(file),
     as.integer(cpl),
     as.integer(1),
     err=as.integer(1),
     PACKAGE="GeneR")$err
  
  
}
