\name{GeneR}
\alias{GeneR}
\title{Overview of GeneR package}
\description{
  GeneR packages allow direct use of nucleotide sequences within R software. 
  Functions can be used to read and write  sequences from main file
  formats (Embl, Genbank and Fasta) in order to perform a lot of
  manipulations and analyses. Main functions are implemented with  C
  extensions.
}

\details{


  To summarize, we can split major functions into 4 categories.

  \bold{ I- Reading and writing sequences}

  GeneR has been designed for fast sequence retrieving even from very
  large sequence databanks, in Fasta, Embl or Genbank formats. It is
  also possible to enter sequences directly from a R command line.  

  There are two ways to store sequences :
  \itemize{
    \item In a C adapted class (buffers) that stores in addition some globals
    variables, like working strand, size of original sequence and so
    on.
  
    It is usefull when, for example, we have to work on a subset of a
    whole chromosome (i.e. a gene). In this case it will be worthwhile
    to load only the gene in R. Nevertheless, it will remain easy to
    associate positions on chromosome and positions on gene \dots

    A complete description on this C class is given in page  \code{\link{globals}}.

    \item As a character string, the more logical way to store short sequences
    like "ATGTCGTG".  It concerns all functions like "strxxx" (strComp,
    strMultiExtract etc.).
  }
  
  \bold{ II- Handling sequences}
  
  The usual copy-paste of parts of 
  sequences or other manipulations can be performed by functions using vectors
  of strands and positions.
  Annotations from the features field within formatted sequence entries can be extracted and used directly in vectors. By this way, it is easy to extract sequence fragments of interest.

  \bold{   III-  Analyzing sequences }
  
  Some tools are designed to count oligo-nucleotides, to look for exact word positions or to shuffle sequences (useful for statistical validations).

  \bold{ IV- Genetic tools }
  
  Finally, the package also contains functions related to genetic and structural aspects of the sequences : ORF localization, translation,
 or RNA secondary structure determination
  (with extension of GeneR: GeneRfold package).

  
}

\seealso{
  \code{\link{globals}},
  \code{\link{readSeq}}, 
  \code{\link{readEmblLocation}},
  \code{\link{getOrfs}}
}


\examples{
## First of all you can try
demo(GeneR)

\dontrun{
demo(hugeGeneR)
}

}

\keyword{utilities}

