\name{bankSummary}
\alias{bankSummary}

\title{Computes informations on all a bank file}
\description{
  This functions computes informations (with function compoSeq,
  GCcontent, of seqSkew) on a complete bank file.
  
  It returns a data frame with one line by sequences in the bank file.

  Parameter "name" can be use to limit the exploration to only a
  few sequences.
}
\usage{
bankSummary(file, name = NULL, type = "F", fun = compoSeq, seqno = 0)
}

\arguments{
  \item{file}{Integer/scalar, File name of the bank (fasta file)}
  \item{seqno}{Integer/scalar, Sequence number (buffer number)}
  \item{name}{String/vector, Names of sequences in bank file, NULL: uses
    all sequences of the bank}
  \item{type}{String/scalar, Bank format ("F" -> fasta. "E" -> embl, "G" -> GenBank)}
  \item{fun}{Function, Function to be used (for example seqSkew}

  
}

\value{
  a data frame with one line by sequence.
}

\keyword{utilities}

\author{Antoine Lucas}

\seealso{\code{\link{bankDensityProfile}}}
\examples{
  for(i in 1:8)
  {
    s=randomSeq(n=100)
    placeString(s,seqno=0)
    writeFasta("toto_norm.fa",append=TRUE,name=i)
  }
  bankSummary(file="toto_norm.fa")
  bankSummary(file="toto_norm.fa",fun=seqSkew)
}

