\name{exactWord}
\alias{exactWord}

\title{Exact matches of an oligomer}
\description{
  Gets match positions of an oligomer in fragments of a sequence
}
\usage{
exactWord(word, seqno=0, from=1, to=0, strand = getStrand(), step=1,
  overlap=TRUE, wNbOcc =-1,case.sensitive=FALSE)

}
\arguments{
 \item{word}{string/scalar, Oligomer sequence }
 \item{seqno}{Integer/scalar, Sequence number (buffer number)}
 \item{from, to}{Integer/scalar, Absolute addresses of the beginning and the end of the fragment,
  (1 means the first nucleotide and 0 conventionally the last one; 
  from must not be larger than to and both vectors must be the same size)}
\item{step}{Integer/scalar, Size of the step in which search progression
  is done}
\item{overlap}{TRUE: look for overlapping oligomer}
 \item{wNbOcc}{Integer/scalar, Maximum number of occurences to retrieve sequentially in each fragment
  (-1 -> all) }
 \item{strand}{Integer/scalar, Strand (forward: 0, reverse: 1)}
 \item{case.sensitive}{T -> difference between lower and upper case letter, F -> no difference}.

}
\value{
  \item{A list of match positions in each fragment.}
  \item{If error : NULL}
}

\note{step with negative values will be implemented soon}

\author{L.Cottret }

\keyword{utilities}


\seealso{\code{\link{getSeq} }}

\examples{
s<-"cgtagctagctagctagctagctagctagcta"
placeString(s)
exactWord(word="ag", from=c(3,11,23),to=c(9,17,29))
#[[1]]
#[1] 4 8
#
#[[2]]
#[1] 12 16
#
#[[3]]
#[1] 24 28

placeString("TTTTTTTTTTTT")
exactWord("TTT")
exactWord("TTT",overlap=FALSE,step=2)

  
}

