

\name{makeIndex}
\alias{makeIndex}
\title{Index file creation for a bank file (internal function)}
\description{Makes an index for a bank file (if there wasn't). }

\usage{makeIndex(file, type="E", index="ix")}



\arguments{
  \item{file}{string/scalar, file name of bank file}
  \item{type}{Sequence format ("E" for Embl, "G", for Genebank, "F" for Fasta)}
  \item{index}{Suffix for the index file (usually: ix)}
}

\value{
  \item{1}{Index already exists (and no changes)}
  \item{0}{Index successfully created}
  \item{-1}{Error}
}

\details{
  Checks if index file exists and is newer than bank file. If not,
  calls one of indexFasta, indexEmbl, indexGbk functions.
  
}
  
\note{Index files are in the form:
  \preformatted{
Accno   deb_feature   deb_sequence   length_sequence
    }
    with one line by sequence. Number of char must be
    the same for each line (it is used to search
    a specific access number) but size used for accno is
    of 40 char by default. This can be change by setting
    variable  \code{MAX\_LEN\_ACCNO} in GeneR\_globals.h file and
    recompile the library. 
}

\author{Antoine Lucas}

\seealso{\code{\link{indexFasta}},\code{\link{indexEmbl}},
  \code{\link{indexGbk}}} 


\keyword{utilities}

\examples{
  seqNcbi("BY608190",file="BY608190.fa")

  # Write index file BY608190.fa.ix ...
  indexFasta("BY608190.fa")

  }

