\name{Read location}
\alias{readGbkLocation}
\alias{readEmblLocation}

\title{Get annotations of a GeneBank or an EMBL sequence}

\description{Extracts by keywords, from a GeneBank or an EMBL sequence entry, annotations
from the features field}

\usage{
readGbkLocation (file, name = NA, from = 1, to = 0, key = "CDS", subkey
                 = "")
readEmblLocation (file, name = NA, from = 1, to = 0, key = "CDS", subkey
                 = "")

}

\arguments{
  \item{file}{String/scalar, File name of the bank}

  \item{name}{String/scalar, Sequence name (default : first sequence)}

  \item{from, to}{Integer/scalar, Absolute addresses of the begin and the end of the fragment,
  (1 means the first nucleotide and 0 conventionally the last one; 
  from must not be larger than to) }
  
  \item{key}{String/scalar,. Feature to retrieve. (ex: GENE, CDS...)}
 
  \item{subkey}{String/scalar, Label to retrieve (ex: locus\_tag, note, codon\_start\dots).}
}

\value{
  A data frame
  }

\author{P.Durosay(C), L.Cottret(R), A. Lucas}

\keyword{utilities}
\keyword{database}

%\note{Use this function carefully: bug found...}

\seealso{\code{\link{readEmblLocation}},\code{\link{readSeq}}} 

\examples{

\dontrun{
# Get SARS Sequence
seqNcbi("NC_004718",file="NC_004718.gbk",type="G")

# Get CDS Positions

readGbkLocation(file="NC_004718.gbk")

# Get peptides...
readGbkLocation(file="NC_004718.gbk",key="mat_peptide",subkey="product")
}

# The same with EMBL...
#
# Get SARS Sequence

\dontrun{
download.file("http://bioinfo.hku.hk/sars/AY291451.seq",
destfile="AY291451.seq")


# Get CDS Positions
readEmblLocation(file="AY291451.seq")

# Get peptides...
readEmblLocation(file="AY291451.seq",key="mat_peptide",subkey="product")
}

}


