/*! \file GeneR_seq.h
 *  \brief sequence class for GeneR
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2006-02-16 16:52:00 lucas>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include "GeneR_globals.h"



/** \class GeneR_seq 
 * \brief a class to manage one sequence and its complementaire
 */
class GeneR_seq
{

 public:
  char *seq;
  char *seqComp;

  size_t  sSize;
  size_t  sSizeComp;

  int BegSeq;
  int MasterSize;
  char AccN[MaxLNom];

  /** \var  seq  
   *  \brief sequences
   */
  /** \var sSize
   * \brief  size of sequences
   */
  /** \var  seqComp  
   *  \brief reverse complementary sequences
   */
  /** \var sSizeComp
   * \brief  size of seqComp
   */
  /** \var BegSeq
   * \brief  begin position in source sequence (i.e. when  we load a part of
   * a sequence in a bank file, by example a subpart of a chromosome).
   */
  /** \var MasterSize
   * \brief  size of source sequence (i.e. when  we load a part of
   * a sequence in a bank file, by example a subpart of a chromosome).
   */
  /** \var AccN
   * \brief Name of sequence. By default: Seq_R
   */


 
  /** \brief Contructor: initialize a sequence. (do not allocate 
   * the sequence)*/
 GeneR_seq()
    {
      seq = NULL;
      seqComp = NULL;
      sSize = 0;   /* size = length allocated = length of seq +1*/ 
      sSizeComp = 0;
      BegSeq = -1;
      MasterSize = -1;
      AccN[0] ='\0';
      /*      printf("c");*/
     }

 /** \fn  GeneR_seq
  * \brief  copy GeneR_seq from itself (just copy the structure;
  * this function is called by function new)
  *
  */
  GeneR_seq(const GeneR_seq & tmpseq )
    {
      seq = NULL;
      seqComp = NULL;
      sSize = 0;   /* size = length allocated = length of seq +1*/ 
      sSizeComp = 0;
      BegSeq = -1;
      MasterSize = -1;
      AccN[0] ='\0';
      /*      printf("r");*/
     }

  /** \brief  Destructor */
  ~GeneR_seq()
    {
      if(seq != NULL)
	free(seq);
      if(seqComp != NULL)
	free(seqComp);

	  /*      printf("o");*/
    }


  /** \fn freeSeq()
   * \brief Free sequence and complementaire. Re init all values.
   */
  void freeSeq()
    {
      if(sSize!=0)
	{
	  free(seq);
	  free(seqComp);
	  seq = NULL;
	  seqComp = NULL;
	  sSize = 0;   /* size = length allocated = length of seq +1*/ 
	  sSizeComp = 0;
	  BegSeq = -1;
	  MasterSize = -1;
	  AccN[0] ='\0';
	}
    }
  
  /** \fn freeCompSeq()
   * \brief Free sequence complementaire.
   */
  void freeCompSeq()
    {
      if(sSizeComp!=0)
	{
	  free(seqComp);
	  sSizeComp = 0;
	  seqComp = NULL;
	}
    }
  
/** @brief allocate a buffer
    @param size  Size to allocate
    @param fill  Caractere de remplissage. fill=0: pas de remplissage
    @param comp  0 : brin sens -  1 : brin anti-sens
    Return 0: error; 1: OK
*/
  int allocBuffer(int size, int comp, int fill);
};
